// Persistence of Vision Ray Tracer Scene Description File
// File:  butterfl.inc
// Vers:  3.0
// Date:  15 April 1997
// Desc:  Butterfly Include-File
// Auth:  Peter Engels
// EMail: engelspj@aol.com
// WWW:   http://www/members.aol.com/engelspj

/*

 Documentation:
 --------------

 This is the first release of my butterfly include file, please send me back
 changes you made, especially if you created new wing-types.

 Mail-address:  engelspj@aol.com
 HomePage    :  http://www/members.aol.com/engelspj



 How to create nice Butterflys in POV-Ray using butterfl.inc
 ===========================================================


 Simply add an include in your pov-file like:
 (be sure that butterfly.inc is in your include-path (set -i option))

    #include "butterfl.inc"

 Then place the butterfly-object that was declared in the inc file into your scene:
 (The butterfly can be rotated, translated and scaled with every value.)

    object {
    ButterFly
    scale     <x-val,y-val,z-val>
    rotate    <x-val,y-val,z-val>
    translate <x-val,y-val,z-val>
           }

 The butterfly is located at the origin (0,0,0) laying on the x-z axis.
 The head is pointing into +z direction, the wings are spread to -x/+x.
 The unscaled butterfly covers the following space:

 x: -1.0   to   +1.0   (depending on WingType and WingAngle)
 y: -0.3   to   +0.2   (if WingAngle = 0  <default>)
 z: -1.0   to   +1.0   (depending on WingType)

 By default following parameters are set (if not declared in the calling pov-file):
 (you can change them simply by declaring before calling butterfl.inc)
 Please take care of the typing ( capital/small letters) i.e: #declare WingType=3

Var:           Type:     Default:          Value:           Description:
----------------------------------------------------------------------------------------------------
WingType       numeric   0                 1 to 5 in V1.0   form of wings (prism-objects)
WingAngle      numeric   0                 -90 .. +90       angle in degrees (0=open)

WingText       texture   Blue              any texture      texture of wings (I prefer image_maps)
EyeText        texture   Gray20 specul .5  any texture      eyes texture (good with specular amount)
BodyText       texture   Gray40 bumps .51  any texture      bodys texture
LegText        texture   Gray10 phong .4   any texture      legs texture (little phong looks good)

LegThickn      numeric   0.02              any number       thickness of legs (change in small steps)

Flying         bool      0                 0 / 1            appearence of legs 0=staying 1=flying

WithoutShadow  bool      0                 0 / 1            1=butterfly casts no shadow

FastButterfly  bool      0                 0 / 1            1=Butterfly is created by
                                                            simple shapes (good for developement)

****************************************
  How to create an additional WingType
****************************************
To create additional WingTypes just create the left wing by declaring a prism
starting at x=0 expanding to the -x direction.
The right Wing is automatically created by butterfl.inc, so dont think about it.

Wing-Example:
=============
#if (WingType=xx) <--- insert here the WingType value you wish
  #declare Wing=
   prism {
    linear_sweep
    linear_spline
    0,                 // sweep the following shape from here ...
    0.001,             // ... up through here
    8,                 // the number of points making up the shape ...
    <0.05,-.3>,<0,0>,<0,.2>,<-.7,.8>,<-.7,-.8>,<0,-.2>,<0,0>,<0.05,-.3>
    texture {WingText}
          }
#end

Insert the new WingType at the end of the inc-file but before the line: #declare ButterFly=
For further information on prism-objects read the POVRAY 3.01 documentation.

*********************************************************************
**    Feel free to modify the existing original WingTypes,         **
**    but leave the existing ones unchanged for compatibility.     **
**    please send me your new WingTypes via mail.                  **
**    Also created pictures and scene-files are welcome            **
**    Ill put the best ones onto my homepage for public access    **
*********************************************************************

**** end of documentation ****/


 #include "colors.inc"

 //  first declare some defaults if not already done in scene script.

 #ifndef (Flying)        #declare Flying        = 0    #end
 #ifndef (WingType)      #declare WingType      = 0    #end
 #ifndef (WingAngle)     #declare WingAngle     = 0    #end
 #ifndef (LegThickn)     #declare LegThickn     = 0.02 #end
 #ifndef (WithoutShadow) #declare WithoutShadow = 0    #end
 #ifndef (FastButterfly) #declare FastButterfly = 0    #end

 #ifndef (WingText)
   #declare WingText = texture{pigment {Blue}finish {ambient 0.3}}
 #end

 #ifndef (BodyText)
   #declare BodyText = texture{pigment{Gray40}finish {ambient 0.3}normal{bumps 0.51 scale 0.005}}
 #end

 #ifndef (LegText)
  #declare LegText   = texture{pigment{Gray10} finish {ambient 0.3 phong .4}}
 #end

#ifndef (EyeText)
  #declare EyeText   = texture{pigment {color Gray20} finish{specular 0.5 }}
#end


//  declare the legs for a staying butterfly  (only if Flying=0)

#if(Flying=0)
#declare Legs=object{

// ****** front
union{
union{
cylinder {<0,.05,0> , <-.2,-.20,.10> , LegThickn}
sphere {<-.2,-.20,0.10>,LegThickn}
cylinder {<-.2,-.20,0.10> , < -.2,.1,0.2> , LegThickn}
sphere {<-.2,.1,0.2>,LegThickn}
cone {<-.2,.1,.2> , LegThickn <-.2,-.55,.4> , LegThickn/2}
scale 0.3
translate <0,-.05,.1>
}

// ****** middle
union{
cylinder {<0,.05,0> , <-.2,-.20,-.10> , LegThickn}
sphere {<-.2,-.20,-0.10>,LegThickn}
cylinder {<-.2,-.20,-0.10> , < -.2,.1,-0.2> , LegThickn}
sphere {<-.2,.1,-0.2>,LegThickn}
cone {<-.2,.1,-.2> , LegThickn <-.2,-.50,-.45> , LegThickn/2}
scale 0.3
translate <0,-.05,.025>
}

// ****** rear
union{
cylinder {<0,.05,0> , <-.2,-.20,-.10> , LegThickn}
sphere {<-.2,-.20,-0.10>,LegThickn}
cylinder {<-.2,-.20,-0.10> , < -.2,.1,-0.2> , LegThickn}
sphere {<-.2,.1,-0.2>,LegThickn}
cone {<-.2,.1,-.2> , LegThickn <-.2,-.5,-.65> , LegThickn/2}
scale 0.3
translate <0,-.05,-.1>
}
}
  }
#end

// declare the legs for a flying butterfly (if Flying=1)

#if (Flying=1)//  ********** Legs flying
#declare Legs=object{
// ****** front
union{
union{
cylinder {<0,.05,0> , <-.1,-.10,.0> , LegThickn}
sphere {<-.1,-.10,0.0>,LegThickn}
cylinder {<-.1,-.10,0.0> , < -.1,.0,0.20> , LegThickn}
sphere {<-.1,.0,0.20>,LegThickn}
cone {<-.1,.0,.20> , LegThickn <-.2,-.25,-.3> , LegThickn/2}
scale 0.3
translate <0,-.05,.1>
}

// ****** middle
union{
cylinder {<0,.05,0> , <-.2,-.15,-.15> , LegThickn}
sphere {<-.2,-.15,-0.15>,LegThickn}
cylinder {<-.2,-.15,-0.15> , < -.2,.1,-0.25> , LegThickn}
sphere {<-.2,.1,-0.25>,LegThickn}
cone {<-.2,.1,-.25> , LegThickn <-.2,-.10,-.9> , LegThickn/2}
scale 0.3
translate <0,-.05,.025>
}

// ****** rear
union{
cylinder {<0,.05,0> , <-.2,-.15,-.15> , LegThickn}
sphere {<-.15,-.15,-0.15>,LegThickn}
cylinder {<-.15,-.15,-0.15> , < -.15,.1,-0.25> , LegThickn}
sphere {<-.15,.1,-0.25>,LegThickn}
cone {<-.15,.1,-.25> , LegThickn <-.15,-.10,-.9> , LegThickn/2}
scale 0.3
translate <0,-.05,-.1>
}
}
  }
#end

// declare the body
#declare Body=
  object{
   union{
     sphere{<0,0,0.25>,0.055 scale <0.6,1,1>}            // head

     // antennas
        union {cylinder{0*z,0.6*z,0.0055} cylinder{0.55*z,0.65*z,0.010}
        rotate<-45,-20,0> translate <-0.005,0.01,0.25> texture {LegText}} // antenna left

     union {cylinder{0*z,0.6*z,0.0055} cylinder{0.55*z,0.65*z,0.010}
        rotate<-45, 20,0> translate < 0.005,0.01,0.25> texture {LegText}} // antenna right

     // eyes l/r
     sphere{<0,0,0>,0.025 scale <0.7,1.3,0.9> translate <-0.023,0.005,0.25> texture {EyeText}} //  eye left
     sphere{<0,0,0>,0.025 scale <0.7,1.3,0.9> translate < 0.023,0.005,0.25> texture {EyeText}} //  eye right

     // body
     sphere{0, 0.065 scale <0.6, 1.1, 5.0> translate z*-0.05}  // body front
     sphere{0, 0.050 scale <0.8, 1.0, 5.5> translate z*-0.43}  // body rear

object {Legs texture {LegText}}
object {Legs texture {LegText} scale <-1,1,1>}

     texture {BodyText}
     }
}


// declare a simple body if FastButterfly=1
#if (FastButterfly=1)
  #declare WingText = texture {pigment {color Red}finish{ambient 0.6}}
  #declare WingType = 0
  #declare BodyText = texture {pigment {color Blue}finish{ambient 0.4}}
  #declare Body     = union{cylinder{z*0.4,z*-0.6,0.07}
                            sphere  {z*0.4,0.1}
                            texture {BodyText}
                           }
#end


//   *******  now declare the wings  depending on WingType   *********

#if (WingType=0)
  #declare Wing=
   prism {
    linear_sweep
    linear_spline
    0,                 // sweep the following shape from here ...
    0.001,             // ... up through here
    8,                 // the number of points making up the shape ...
    <0.05,-.3>,<0,0>,<0,.2>,<-.7,.8>,<-.7,-.8>,<0,-.2>,<0,0>,<0.05,-.3>
    texture {WingText}
      }
#end

#if (WingType=1)
  #declare Wing=
   prism {
    linear_sweep
    cubic_spline
    0,                 // sweep the following shape from here ...
    0.001,             // ... up through here
    9,                 // the number of points making up the shape ...
    <0.05,-.1>,<0,-.05>,<-.10,.2>,<-.6,.3>,<-.9,0>,<-.09,-.9>,<-.05,-.3>,<0,-.05>,<0.05,-.1>
   texture {WingText}
        }
#end

#if (WingType=2)
  #declare Wing=
   prism {
    linear_sweep
    cubic_spline
    0,                 // sweep the following shape from here ...
    0.001,             // ... up through here
    9,                 // the number of points making up the shape ...
    <0.05,-.3>,<0,-.05>,<-.02,.1>,<-.7,.8>,<-.6,0>,<-.6,-.8>,<0,-.2>,<0,-.05>,<0.05,-.3>
   texture {WingText}
        }
#end

#if (WingType=3)
  #declare Wing=
   prism {
    linear_sweep
    cubic_spline
    0,                 // sweep the following shape from here ...
    0.001,             // ... up through here
    9,                 // the number of points making up the shape ...
    <0.05,-.3>,<0,0>,<-.1,.2>,<-.9,.7>,<-.8,0>,
    <-.08,-.99>,<-.15,-.35>,<0,0>,<0.05,-.3>
   texture {WingText}
        }
#end

#if (WingType=4)
  #declare Wing=
   prism {
    linear_sweep
    cubic_spline
    0,                 // sweep the following shape from here ...
    0.001,             // ... up through here
    17,                // the number of points making up the shape ...
    <0.05,-.3>,<0,0>,<-.1,.2>,<-.25,.5>,<-.9,.7>,<-.7,.45>,<-.45,0>,
    <-.58,-.35>,<-.55,-.55>,<-.45,-.75>,<-.425,-.9>,<-.4,-.7>,<-.3,-.6>,<-.2,-.4>,<-.1,-.3>,<0,0>,<0.05,-.1>
   texture {WingText}
        }
#end

#if (WingType=5)
  #declare Wing=
   prism {
    linear_sweep
    cubic_spline
    0,                 // sweep the following shape from here ...
    0.001,             // ... up through here
    17,                // the number of points making up the shape ...
    <0.05,-.3>,<0,0>,<-.1,.2>,<-.25,.5>,<-.7,.7>,<-.7,.45>,<-.55,0>,
    <-.6,-.35>,<-.55,-.55>,<-.45,-.75>,<-.225,-.9>,<-.3,-.7>,<-.175,-.6>,<-.15,-.4>,<-.1,-.3>,<0,0>,<.05,-.1>
   texture {WingText}
        }
#end


/**********************************************
 *      insert new WingType here   !!!!!!     *
 **********************************************/


// now put together the body and the two wings and declare the ButterFly object

#declare ButterFly=
union{
      object {Body}
      object {Wing                rotate -z*WingAngle}
      object {Wing scale <-1,1,1> rotate  z*WingAngle}

  #if (WithoutShadow=1)
    no_shadow
  #end
     }

//  *********** end of Butterfl.inc     (c) Peter Engels
